// Copyright (C) 1998 Ariel Tankus
// 
// This file is part of the Image Processing Toolbox for Octave.
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//

// Author: Ariel Tankus <arielt@math.tau.ac.il>
// Created: 30 May 1998.
// Version: 1.0

#include <octave/oct.h>
#include <octave/error.h>

//#define DEBUG_CONV2

#undef BOUNDS_CHECKING

#undef MIN
#undef MAX
#define MIN(a,b)	(((a) <= (b)) ? (a) : (b))
#define MAX(a,b)	(((a) <= (b)) ? (b) : (a))

//
// setZeroMatrix:
//
// Set all elements of matrix `m' to zero.
//
void setZeroMatrix(Matrix& m)
{
    register int row, col;
    register int mRows = m.rows();
    register int mCols = m.cols();

    for (row = 0; row < mRows; row++)
	for (col = 0; col < mCols; col++)
	    m(row, col) = 0;
}



//
// convolveSame:
//
// Convolve two matrices `mat1' and `mask' in the region whose size is the
// size of `mat1'.
//
void convolveSame(const Matrix& mat1, const Matrix& mask, Matrix& result)
{
    setZeroMatrix(result);

    int r, c, dr, dc;		// iterators
    int maskRowSize, maskColSize;	// dims of mask for current iteration

    for (r = 0; r < mat1.rows(); r++) {

	// index of last row in current mask
	maskRowSize = MIN(mask.rows(), mat1.rows() - r);

	for (c = 0; c < mat1.cols(); c++) {

	    // index of last column in current mask
	    maskColSize = MIN(mask.cols(), mat1.cols() - c);

	    // iterate on mask elements: (dr,dc)
	    for (dr = 0; dr < maskRowSize; dr++)
		for (dc = 0; dc < maskColSize; dc++) {

		    // convolve mat1 and mask
		    result(r, c) += mat1(r+dr, c+dc) *
			mask(mask.rows()-1-dr, mask.cols()-1-dc);
		    
		}
	}
    }
}

//
// convolveValid:
//
// Convolve two matrices `mat1' and `mask' in the region where no compact
// support is necessary for the convolution, i.e. the result is of size
// (mat1.rows() - mask.rows() + 1) x (mat1.cols() - mask.cols() + 1)
//
void convolveValid(const Matrix& mat1, const Matrix& mask, Matrix& result)
{
    setZeroMatrix(result);

    // #rows,#cols where conv is valid (i.e. w/o compact support)
    int numRowsValid = mat1.rows() - mask.rows() + 1;
    int numColsValid = mat1.cols() - mask.cols() + 1;

    int r, c, dr, dc;		// iterators
    int maskRowSize, maskColSize;	// dims of mask for current iteration

    // iterate on mat1 elements: (r,c)
    for (r = 0; r < numRowsValid; r++)
	for (c = 0; c < numColsValid; c++)

	    // iterate on mask elements: (dr,dc)
	    for (dr = 0; dr < mask.rows(); dr++)
		for (dc = 0; dc < mask.cols(); dc++) {

		    // convolve mat1 and mask
		    result(r, c) += mat1(r+dr, c+dc) *
			mask(mask.rows()-1-dr, mask.cols()-1-dc);
		    
		}
}


//
// convolveFull:
//
// Convolve two matrices `mat1' and `mask' in the region whose size is the
// size of `mat1'.

//
void convolveFull(const Matrix& mat1, const Matrix& mask, Matrix& result)
{
    setZeroMatrix(result);

    int r, c, matR, matC;			// iterators.

    // optimization variables:
    const double *dMat1 = mat1.data();
    const double *dMask = mask.data();

    int mat1Rows = mat1.rows();
    int mat1Cols = mat1.cols();

    int maskRows = mask.rows();
    int maskCols = mask.cols();

    int resultRows = mat1Rows + maskRows - 1;
    int resultCols = mat1Cols + maskCols - 1;
    double *dResult = result.fortran_vec();

    register double tmp;
    const double *mat1Ptr, *maskPtr;

    // The following is a non-optimized version of the code:
    //
    //for (r = 0; r < mat1Rows+maskRows-1; r++)
    // 	for (c = 0; c < mat1Cols+maskCols-1; c++) {
    //
    // 	    for (matR = MAX(r - maskRows+1, 0); matR < MIN(r+1, mat1Rows);
    // 		 matR++)
    // 		for (matC = MAX(c - maskCols+1, 0); matC < MIN(c+1, mat1Cols); 
    // 		     matC++) {
    // 		    // convolve mat1 and mask
    // 		    result(r, c) += mat1(matR, matC) * mask(r-matR, c-matC);

    // matRLow and matRHigh (see below) are derived from the constraints:
    // 0 <= r - matR < mask.rows()
    // 0 <= matR < mat1.rows()
    //
    // Similar constraint hold for matCLow, matCHigh.

    // The optimized code:
    for (r = 0; r < mat1Rows+maskRows-1; r++)
	for (c = 0; c < mat1Cols+maskCols-1; c++) {

	    int matRLow  = MAX(r - maskRows+1, 0);
	    int matRHigh = MIN(r+1, mat1Rows);

	    int matCLow  = MAX(c - maskCols+1, 0);
	    int matCHigh = MIN(c+1, mat1Cols);

	    tmp = 0;

	    for (matC = matCLow; matC < matCHigh; matC++) {

		// optim.: reduce #operations in address calc.
		mat1Ptr = dMat1 + mat1Rows*matC;
		maskPtr = dMask + r + maskRows * (c - matC);

		for (matR = matRLow; matR < matRHigh; matR++) {
		    // convolve mat1 and mask
		     tmp += (*(mat1Ptr + matR)) * (*(maskPtr - matR));

#ifdef DEBUG_CONV2
		    warning("c = %d   ; r = %d", c, r);
		    warning("matC = %d; matR = %d", matC, matR);
		    warning("result(%d,%d) += mat1(%d,%d)*mask(%d,%d)",
			    r, c, matR, matC, r - matR, c - matC);
		    warning("%g = %g * %g", result(r, c), mat1(matR, matC),
			    mask(r - matR, c - matC));
#endif
		}
	    }

	    *(dResult + r + resultRows*c) = tmp;
	}
}


DEFUN_DLD (conv2, args, nargout, "Two dimensional convolution.")
{
    // check input arguments
    if (args.length() < 2) {
	if (args.length() == 1)
	    error("conv2 requires at least two inputs: given one input.");
	else
	    error("conv2 requires at least two inputs: given %d inputs.",
		  args.length());
	return octave_value();
    }

    if (!args(0).is_real_matrix()) {
	error("conv2 requires argument #1 to be a real matrix."); 
	return octave_value();
    }	

    if (!args(1).is_real_matrix()) {
	error("conv2 requires argument #2 to be a real matrix."); 
	return octave_value();
    }	

    Matrix mat1 = args(0).matrix_value();
    Matrix mask = args(1).matrix_value();

    if ((mask.rows() > mat1.rows()) || (mask.cols() > mat1.cols())) {
	error("mask matrix dimensions should be smaller than those of original matrix");
	return octave_value();
    }

    // define and init results matrix

    // check whether type of convolution given by user.
    charMatrix convolveDims;
    if (args.length() < 3)
	convolveDims = charMatrix("same");		// default value

    if (args.length() >= 3) {
	if (!args(2).is_char_matrix()) {

	    error("conv2 requires argument #3 to be a char matrix.");
	    return octave_value();

	} else

	    convolveDims = args(2).char_matrix_value();

    }

    if (convolveDims == charMatrix("same")) {

	Matrix result(mat1.rows(), mat1.cols());
	convolveSame(mat1, mask, result);
	return octave_value(result);

    } else if (convolveDims == charMatrix("valid")) {

	Matrix result(mat1.rows()-mask.rows()+1,mat1.cols()-mask.cols()+1);
	convolveValid(mat1, mask, result);
	return octave_value(result);

    } else if (convolveDims == charMatrix("full")) {

	Matrix result(mat1.rows()+mask.rows()-1, mat1.cols()+mask.cols()-1);
	convolveFull(mat1, mask, result);
	return octave_value(result);

    } else {

	error("Illegal convolution type (3rd argument) given to conv2.");
	return octave_value();

    }

}
