## Copyright (C) 1998 Ariel Tankus
## 
## This program is free software.
## This file is part of the Image Processing Toolbox for Octave
##
## This program is free software; you can redistribute it and/or
## modify it under the terms of the GNU General Public License
## as published by the Free Software Foundation; either version 2
## of the License, or (at your option) any later version.
## 
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
## 
## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
##

## readimagefile    Read an image file of any supported image format.
##                  The currently supported formats are:
##                  PGM/PPM (both ascii and binary), FI, JFIF.
##
##                  im = readimagefile(fname, force_gray)
##                  fname      - string - name of image file.
##                  force_gray - flag   - (opt) if true, the returned image
##                               is a gray level image, even if the file
##                               contianed a color image. (default: flase)
##                  im         - matrix - an image.  Note, that if
##                               force_gray == false, the image might by a 
##                               PPM representation of the image.
##                  isPpm - opt. - 1 - `im' is in PPM format.
##                                 0 - `im' is a graylevel matrix.
##
##                  See also: readpnm, readfi, readjpg, splitppm.
##

## Author: Ariel Tankus <arielt@math.tau.ac.il>
## Created: August 1998
## Version: 1.0

function [im, isPpm] = readimagefile(fname, force_gray)

if (nargin < 2)
    force_gray = 0;
end

fname = file_in_path(IMAGEPATH, fname);

[fid, magicnum] = getmagicnum(fname);
fclose(fid);

if (nargout > 1)
    isPpm = 0;
end

if (strcmp(magicnum, 'P2') | strcmp(magicnum, 'P5'))
    ## PGM
    im = readpnm(fname);
elseif (strcmp(magicnum, 'P3') | strcmp(magicnum, 'P6'))
    ## PPM
    im = readpnm(fname);
    if (force_gray)
        [r,g,b] = splitppm(im);
        im = (r + g + b) / 3;
    end
    if (nargout > 1)
        isPpm = 1;
    end
    
elseif (strcmp(magicnum, 'GI'))
    error('Probably a GIF file. Currently not supported.');
elseif (strcmp(magicnum, ''))
    [im, isPpmTmp] = readjpg(fname);

    if (isPpmTmp & force_gray)
        [r,g,b] = splitppm(im);
        im = (r + g + b) / 3;        # create gray-level image
        isPpmTmp = 0;
    end
    if (nargout > 1)
        isPpm = isPpmTmp;
    end

elseif (strcmp(magicnum, 'FI'))
    im = readfi(fname);
else
    error('Unknow file type encountered.');
end

endfunction
