## Copyright (C) 1998 Ariel Tankus
## 
## This program is free software.
## This file is part of the Image Processing Toolbox for Octave
##
## This program is free software; you can redistribute it and/or
## modify it under the terms of the GNU General Public License
## as published by the Free Software Foundation; either version 2
## of the License, or (at your option) any later version.
## 
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
## 
## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
##

## readpnm     Read a PGM or a PPM image file into Octave.  The PGM/PPM
##             image files may be gzipped.  Both binary and ascii PGMs
##             or PPMs can be read.
##
##             im = readpnm(fname)
##             fname - name of PNM file.
##             im    - the image as an Octave variable.
##             isPpm - opt. - 1 - `im' is in PPM format.
##                            0 - `im' is a graylevel matrix.
##
##             im = readpnm(fid)
##             fid - file ID (for the PNM image file).
##             im  - the image as an Octave variable.
##             isPpm - opt. - 1 - `im' is in PPM format.
##                            0 - `im' is a graylevel matrix.
##

## Author: Ariel Tankus <arielt@math.tau.ac.il>
## Created: 8.5.98 .
## Version: 1.0

function [im, isPpm] = readpnm(fname)

## constants:
ascPgmMagic = 'P2';
binPgmMagic = 'P5';
ascPpmMagic = 'P3';
binPpmMagic = 'P6';

[fid, magicnum] = getmagicnum(fname);

width  = str2num(getNonComment(fid, '#'));
height = str2num(getNonComment(fid, '#'));

if (strcmp(magicnum, ascPpmMagic) | strcmp(magicnum, binPpmMagic))
    ## reserve room for color info
    width  = width * 3;
end

if (strcmp(magicnum, ascPgmMagic) |  strcmp(magicnum, ascPpmMagic))

    ## ascii PGM/PPM
    maxval = str2num(getNonComment(fid, '#'));

    ## dummy read next token, to skip the comments
    ## This should handle a comment after `maxval'.
    dummy = getNonComment(fid, '#');

    ## return to position just before last token
    fseek(fid, -length(dummy), SEEK_CUR);

    [data, count] = fscanf(fid, "%d", [width, height]);

elseif (strcmp(magicnum, binPgmMagic) | strcmp(magicnum, binPpmMagic))

    ## binary PGM/PPM image
    [maxval, whitespace] = fscanf(fid, ' %s%c', "C");
    if (~isspace(whitespace))
        error('Illegal character after header.');
    end
    maxval = str2num(maxval);

    ## PGM/PPM binary image
    [data, count] = fread(fid, [width, height], 'uchar');

else

    error('Magic number unknown!');

end

if (count < width*height)
    error('Unable to read the whole image.  File might be truncated.');
end
im = data';		# convert image [x,y] to matrix [m,n]

fclose(fid);

if (nargout > 1)
    isPpm = (strcmp(magicnum, ascPpmMagic) | ...
        strcmp(magicnum, binPpmMagic));
end

endfunction
