## Copyright (C) 1998 Ariel Tankus
## 
## This file is part of Octave.
##
## This program is free software; you can redistribute it and/or
## modify it under the terms of the GNU General Public License
## as published by the Free Software Foundation; either version 2
## of the License, or (at your option) any later version.
## 
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
## 
## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
##

## cart2pol    Convert Cartsian representation to Polar representation.
##
##             Cartesian to Polar:
##
##             [theta, r] = cart2pol(x, y)
##             x     - X coordinate of the vector.
##             y     - Y coordinate of the vector.
##
##             theta - rad - angle from X axis counterclockwise.
##             r     - norm of vector.
##
##             Cartesian to Cylinderical:
##
##             [theta, r, z] = cart2pol(x, y, z)
##             x     - X coordinate of the vector.
##             y     - Y coordinate of the vector.
##             z     - Z coordinate of the vector.
##
##             theta - rad - angle from X axis counterclockwise.
##             r     - norm of the projection on [XY] plane of the vector.
##             z     - Z coordinate of the vector.
##
##             If only one input/output argument exists, it is assumed a
##             vector of length 2 or 3, whose elements are the appropriate
##             coordinates.
##
##             See also: pol2cart, sph2cart, cart2sph.
##

## Author: Ariel Tankus. <arielt@math.tau.ac.il>
## Created: 10.11.98.
## Version: 1.1

function [theta, r, z] = cart2pol(x, y, z)

  is_cylinderic = ((nargin == 3) | ((nargin == 1) & (prod(size(x)) == 3)));
  if ((nargin == 1) & (prod(size(x)) == 2))
    ## planar coordinates
    y = x(2);
    x = x(1);
  end
  if ((nargin == 1) & (prod(size(x)) == 3))
    ## 3D coordinates
    y = x(2);
    z = x(3);
    x = x(1);
  end

  r = sqrt(x^2 + y^2);
  theta = atan2(y, x);

  if (nargout <= 1)
    if (~is_cylinderic)
      theta = [theta, r];       # polar coordinates
    else
      theta = [theta, r, z];    # cylinderical coordinates
    end
  end

endfunction
