## Copyright (C) 1998 Ariel Tankus
## 
## This file is part of Octave.
##
## This program is free software; you can redistribute it and/or
## modify it under the terms of the GNU General Public License
## as published by the Free Software Foundation; either version 2
## of the License, or (at your option) any later version.
## 
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
## 
## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
##

## cart2sph    Convert cartesian coordinates to spheric representation.
##
##             [az, el, r] = cart2sph(x, y, z)
##             x - X coordinate of the vector.
##             y - Y coordinate of the vector.
##             z - Z coordinate of the vector.
##
##             az - rad - azimuth. angle from the X axis to the projection
##                        of the vector on the [XY] plane.
##             el - rad - elevation. angle between  the projection of the
##                        vector on the [XY] plane and the vector itself.
##             r  - norm of vector.
##
##             If only one input/output argument exists, it is assumed a
##             vector of length 3, whose elements are the appropriate
##             coordinates.
##
##             See also: cart2sph, cart2pol, pol2cart.
##

## Author: Ariel Tankus. <arielt@math.tau.ac.il>
## Created: 10.11.98.
## Version: 1.1

function [az, el, r] = cart2sph(x, y, z)

  ## allow `x' to contain the whole vector
  if ((nargin == 1) & (prod(size(x)) == 3))
    y = x(2);
    z = x(3);
    x = x(1);
  end

  r  = sqrt(x^2 + y^2 + z^2);
  az = atan2(y, x);
  if (r ~= 0)
    el = asin(z/r);
  else
    el = 0;                     # r = 0   <=>  x = y = z = 0
  end

  if (nargout <= 1)
    az = [az; el; r];
  end

endfunction
