## Copyright (C) 1998 Ariel Tankus
## 
## This file is part of Octave.
##
## This program is free software; you can redistribute it and/or
## modify it under the terms of the GNU General Public License
## as published by the Free Software Foundation; either version 2
## of the License, or (at your option) any later version.
## 
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
## 
## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
##

## pol2cart    Convert Polar representation to Cartsian representation.
##             Convert Cylinderical representation to Cartsian representation.
##
##             Polar to Cartesian:
##
##             [x, y] = pol2cart(theta, r)
##             theta - rad - angle from X axis counterclockwise.
##             r     - norm of vector.
##
##             x     - X coordinate of the vector.
##             y     - Y coordinate of the vector.
##
##             Cylinderical to Cartesian:
##
##             [x, y, z] = pol2cart(theta, r, z)
##             theta - rad - angle from X axis counterclockwise.
##             r     - norm of projection of the vector on the [XY] plane.
##             z     - vertical axis.
##
##             x     - X coordinate of the vector.
##             y     - Y coordinate of the vector.
##             z     - vertical axis.
##
##             If only one input/output argument exists, it is assumed a
##             vector of length 2 or 3, whose elements are the appropriate
##             coordinates.
##
##             See also: cart2pol, sph2cart, cart2sph.
##

## Author: Ariel Tankus. <arielt@math.tau.ac.il>
## Created: 10.11.98.
## Version: 1.1

function [x, y, z] = pol2cart(theta, r, z)

  is_cylinderic = ((nargin == 3) | ((nargin == 1) & (prod(size(theta)) == 3)));
  if ((nargin == 1) & (prod(size(theta)) == 2))
    ## polar coordinates
    r = theta(2);
    theta = theta(1);
  end
  if ((nargin == 1) & (prod(size(theta)) == 3))
    ## cylinderical coordinates
    r = theta(2);
    z = theta(3);
    theta = theta(1);
  end

  x = r * cos(theta);
  y = r * sin(theta);

  if (nargout <= 1)
    if (~is_cylinderic)
      x = [x, y];
    else
      x = [x, y, z];
    end
  end

endfunction
