## Copyright (C) 1996 John W. Eaton
##
## This file is part of Octave.
##
## Octave is free software; you can redistribute it and/or modify it
## under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2, or (at your option)
## any later version.
##
## Octave is distributed in the hope that it will be useful, but
## WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
## General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with Octave; see the file COPYING.  If not, write to the Free
## Software Foundation, 59 Temple Place - Suite 330, Boston, MA
## 02111-1307, USA.

## usage: [xb, yb] = bar (x, y)
##
## Given two vectors of x-y data, bar produces a `bar' graph.
##
## If only one argument is given, it is taken as a vector of y-values
## and the x coordinates are taken to be the indices of the elements.
##
## If two output arguments are specified, the data are generated but
## not plotted.  For example,
##
##   bar (x, y);
##
## and
##
##   [xb, yb] = bar (x, y);
##   plot (xb, yb);
##
## are equivalent.
##
## See also: plot, semilogx, semilogy, loglog, polar, mesh, contour,
##           stairs, gplot, gsplot, replot, xlabel, ylabel, title

## Author: jwe
##
## Modified by Ariel Tankus, 7.9.98 .
##

function [xb, yb] = bar (x, y)

  if (nargin > 2)
    usage ("[xb, yb] = bar (x, y)");
  endif

  if (nargin == 1)
    if (~is_vector (x))
      error ("bar: argument must be a vector");
    endif

    y = x;
    x = 1:length(y);
  endif


  if (~is_vector (x) || ~is_vector (y))
    error ("bar: arguments must be vectors");
  endif

  ## ensure x is column vector
  if (size(x,2) ~= 1)
    x = x';
  endif

  ## ensure y is column vector
  if (size(y,2) ~= 1)
    y = y';
  endif

  xlen = length (x);
  ylen = length (y);
  if (xlen ~= ylen)
    error ("bar: arguments must be the same length");
  endif

  [x, indices] = sort(x);
  y = y(indices);

  delta = min(diff(x));
  tmp = [x, y, ones(length(x),1)*delta];
  if (ishold == 0)
    gplot [] [min(0,min(y)):] tmp with boxes
  else
    ## can't specify new range if hold on
    gplot tmp with boxes
  endif

endfunction
